"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const joi = require("joi");
const validation_1 = require("../../error/validation");
const _ = require("lodash");
const defaultOptions = {
    stripUnknown: true
};
function createValidationMiddleware(schema) {
    return function (ctx, next) {
        let params;
        if (ctx.request.method === 'GET') {
            params = ctx.request.query;
        }
        else {
            const contentType = ctx.req.headers['content-type'];
            if (!contentType || contentType.indexOf('application/json') > -1) {
                params = ctx.request.body;
            }
            else if (contentType.indexOf('multipart/form-data') > -1 && ctx.request.body.fields) {
                params = ctx.request.body.fields;
            }
            else {
                params = ctx.request.body;
            }
        }
        const result = joi.validate(params, schema, defaultOptions);
        if (result.error) {
            const invalidKey = _.get(result.error, 'details.0.path');
            const invalidValue = params[invalidKey];
            const message = _.get(result.error, 'details.0.message');
            throw new validation_1.ValidationError(validation_1.ValidationError.DEFAULT, invalidKey, invalidValue, message);
        }
        ctx.validatedData = Object.assign({}, params, result.value);
        next();
    };
}
exports.createValidationMiddleware = createValidationMiddleware;
